[project]
name = "{{stinger.name|hyphen_case}}-ipc"
version = "{{stinger.version}}"
description = "{{stinger.title}}"
requires-python = ">=3.9"
dependencies = [
    "pydantic>=2.11.7",
    "paho-mqtt>=2.0.1",
    "isodate>=0.7.2",
]
