
from pydantic import BaseModel
from .interface_types import *

{%if stinger.properties | length > 0 %}
class {{stinger.name|UpperCamelCase}}InitialPropertyValues(BaseModel):
    {%for prop_name, prop in stinger.properties.items()%}
    {{prop_name | snake_case}}: {{prop.python_annotation}}
    {{prop_name | snake_case}}_version: int = 0
    {%endfor%}
{%endif%}