/*
DO NOT MODIFY THIS FILE.  It is automatically generated and changes will be over-written
on the next generation.

It contains enumerations used by the {{stinger.name}} interface.

LICENSE: This generated code is not subject to any license restrictions from the generator itself.
TODO: Get license text from stinger file
*/

#pragma once

{%for ie_name, ie in stinger.enums.items() %}
enum class {{ie_name | UpperCamelCase }} {
    {%-for value in ie.values %}
    {{value | CONST_CASE}} = {{loop.index}}{%if not loop.last%},{%endif%}
    {%-endfor%}
};
static const char *{{ie_name | camelCase }}Strings[] = {"{{ie_name | UpperCamelCase }}", {%for value in ie.values %}"{{value}}"{%if not loop.last%}, {%endif%}{%endfor%} };
{%endfor%}
