@PACKAGE_INIT@
{% set cmake_name %}{{stinger.name|PascalCase|replace('_', '')|replace('--', '')}}{% endset %}

# Ensure consumers get dependencies
include(CMakeFindDependencyMacro)
find_dependency(PkgConfig REQUIRED)
find_dependency(RapidJSON REQUIRED)

pkg_check_modules(mosquitto REQUIRED libmosquitto IMPORTED_TARGET)

# Include targets exported during install
include("${CMAKE_CURRENT_LIST_DIR}/{{cmake_name}}-targets.cmake")
